@@ -47,13 +47,24 @@ struct devcfg_data {
                                            TCP port is open. */
     struct termios termctl;
 
+    void (*shutdown_done)(struct devio *);
+
+    int default_bps;
+    int default_data_size;
+    int default_stop_bits;
+    int default_parity_on;
+    int current_bps;
+    int current_data_size;
+    int current_stop_bits;
+    int current_parity_on;
+
     /* Holds whether break is on or not. */
     int break_set;
 
     /* Disable break-commands */
     int disablebreak;
 
-#ifdef USE_RS485_FEATURE
+#if HAVE_DECL_TIOCSRS485
     struct serial_rs485 *conf;
 #endif
 };
@@ -89,18 +100,20 @@ static struct baud_rates_s {
 #define BAUD_RATES_LEN ((sizeof(baud_rates) / sizeof(struct baud_rates_s)))
 
 static int
-get_baud_rate(int rate, int *val, int cisco)
+get_baud_rate(int rate, int *val, int cisco, int *bps)
 {
     unsigned int i;
     for (i=0; i<BAUD_RATES_LEN; i++) {
 	if (cisco) {
 	    if (rate == baud_rates[i].cisco_ios_val) {
 		*val = baud_rates[i].val;
+		*bps = baud_rates[i].real_rate;
 		return 1;
 	    }
 	} else {
 	    if (rate == baud_rates[i].real_rate) {
 		*val = baud_rates[i].val;
+		*bps = baud_rates[i].real_rate;
 		return 1;
 	    }
 	}
@@ -218,7 +231,6 @@ uucp_mk_lock(char *devname)
 	    if (pid > 0 && kill((pid_t)pid, 0) < 0 && errno == ESRCH) {
 		/* death lockfile - remove it */
 		unlink(lck_file);
-		sleep(1);
 		pid = 0;
 	    } else
 		pid = 1;
@@ -252,6 +264,12 @@ uucp_mk_lock(char *devname)
 
     return pid;
 }
+#else
+static void uucp_rm_lock(char *devname) { }
+static int uucp_mk_lock(char *devname)
+{
+    return 0;
+}
 #endif /* USE_UUCP_LOCKING */
 
 #ifdef __CYGWIN__
@@ -264,24 +282,232 @@ static void cfmakeraw(struct termios *termios_p) {
 }
 #endif
 
+static int
+set_termios_from_speed(struct devcfg_data *d, struct termios *termctl,
+		       int speed)
+{
+    switch (speed) {
+    case 300:
+	cfsetospeed(termctl, B300);
+	cfsetispeed(termctl, B300);
+	break;
+
+    case 600:
+	cfsetospeed(termctl, B600);
+	cfsetispeed(termctl, B600);
+	break;
+
+    case 1200:
+	cfsetospeed(termctl, B1200);
+	cfsetispeed(termctl, B1200);
+	break;
+
+    case 2400:
+	cfsetospeed(termctl, B2400);
+	cfsetispeed(termctl, B2400);
+	break;
+
+    case 4800:
+	cfsetospeed(termctl, B4800);
+	cfsetispeed(termctl, B4800);
+	break;
+
+    case 9600:
+	cfsetospeed(termctl, B9600);
+	cfsetispeed(termctl, B9600);
+	break;
+
+    case 19200:
+	cfsetospeed(termctl, B19200);
+	cfsetispeed(termctl, B19200);
+	break;
+
+    case 38400:
+	cfsetospeed(termctl, B38400);
+	cfsetispeed(termctl, B38400);
+	break;
+
+    case 57600:
+	cfsetospeed(termctl, B57600);
+	cfsetispeed(termctl, B57600);
+	break;
+
+    case 115200:
+	cfsetospeed(termctl, B115200);
+	cfsetispeed(termctl, B115200);
+	break;
+
+#ifdef B230400
+    case 230400:
+	cfsetospeed(termctl, B230400);
+	cfsetispeed(termctl, B230400);
+	break;
+#endif
+#ifdef B460800
+    case 460800:
+	cfsetospeed(termctl, B460800);
+	cfsetispeed(termctl, B460800);
+	break;
+#endif
+#ifdef B500000
+    case 500000:
+	cfsetospeed(termctl, B500000);
+	cfsetispeed(termctl, B500000);
+	break;
+#endif
+#ifdef B576000
+    case 576000:
+	cfsetospeed(termctl, B576000);
+	cfsetispeed(termctl, B576000);
+	break;
+#endif
+#ifdef B921600
+    case 921600:
+	cfsetospeed(termctl, B921600);
+	cfsetispeed(termctl, B921600);
+	break;
+#endif
+#ifdef B1000000
+    case 1000000:
+	cfsetospeed(termctl, B1000000);
+	cfsetispeed(termctl, B1000000);
+	break;
+#endif
+#ifdef B1152000
+    case 1152000:
+	cfsetospeed(termctl, B1152000);
+	cfsetispeed(termctl, B1152000);
+	break;
+#endif
+#ifdef B1500000
+    case 1500000:
+	cfsetospeed(termctl, B1500000);
+	cfsetispeed(termctl, B1500000);
+	break;
+#endif
+#ifdef B2000000
+    case 2000000:
+	cfsetospeed(termctl, B2000000);
+	cfsetispeed(termctl, B2000000);
+	break;
+#endif
+#ifdef B2500000
+    case 2500000:
+	cfsetospeed(termctl, B2500000);
+	cfsetispeed(termctl, B2500000);
+	break;
+#endif
+#ifdef B3000000
+    case 3000000:
+	cfsetospeed(termctl, B3000000);
+	cfsetispeed(termctl, B3000000);
+	break;
+#endif
+#ifdef B3500000
+    case 3500000:
+	cfsetospeed(termctl, B3500000);
+	cfsetispeed(termctl, B3500000);
+	break;
+#endif
+#ifdef B4000000
+    case 4000000:
+	cfsetospeed(termctl, B4000000);
+	cfsetispeed(termctl, B4000000);
+	break;
+#endif
+    default:
+	return -1;
+    }
+
+    d->default_bps = speed;
+    return 0;
+}
+
+static void
+set_termios_parity(struct devcfg_data *d, struct termios *termctl,
+		   enum parity_vals val)
+{
+    switch (val) {
+    case PARITY_NONE:
+	termctl->c_cflag &= ~(PARENB);
+	break;
+    case PARITY_EVEN:
+	termctl->c_cflag |= PARENB;
+	termctl->c_cflag &= ~(PARODD);
+	break;
+    case PARITY_ODD:
+	termctl->c_cflag |= PARENB | PARODD;
+    }
+    d->default_parity_on = val != PARITY_NONE;
+}
+
+static void
+set_termios_xonoff(struct termios *termctl, int enabled)
+{
+    if (enabled) {
+	termctl->c_iflag |= (IXON | IXOFF | IXANY);
+	termctl->c_cc[VSTART] = 17;
+	termctl->c_cc[VSTOP] = 19;
+    } else {
+	termctl->c_iflag &= ~(IXON | IXOFF | IXANY);
+    }
+}
+
+static void
+set_termios_rtscts(struct termios *termctl, int enabled)
+{
+    if (enabled)
+	termctl->c_cflag |= CRTSCTS;
+    else
+	termctl->c_cflag &= ~CRTSCTS;
+}
+
+static void
+set_termios_datasize(struct devcfg_data *d, struct termios *termctl, int size)
+{
+    termctl->c_cflag &= ~CSIZE;
+    switch (size) {
+    case 5: termctl->c_cflag |= CS5; break;
+    case 6: termctl->c_cflag |= CS6; break;
+    case 7: termctl->c_cflag |= CS7; break;
+    case 8: termctl->c_cflag |= CS8; break;
+    }
+    d->default_data_size = size;
+}
+
 /* Initialize a serial port control structure for the first time.
    This should only be called when the port is created.  It sets the
    port to the default 9600N81. */
 static void
-devinit(struct termios *termctl)
+devinit(struct devcfg_data *d, struct termios *termctl)
 {
     cfmakeraw(termctl);
-    cfsetospeed(termctl, B9600);
-    cfsetispeed(termctl, B9600);
-    termctl->c_cflag &= ~(CSTOPB);
-    termctl->c_cflag &= ~(CSIZE);
-    termctl->c_cflag |= CS8;
-    termctl->c_cflag &= ~(PARENB);
-    termctl->c_cflag &= ~(CLOCAL);
-    termctl->c_cflag &= ~(HUPCL);
+
+    set_termios_from_speed(d, termctl, find_default_int("speed"));
+    set_termios_datasize(d, termctl, find_default_int("databits"));
+    d->default_stop_bits = find_default_int("stopbits");
+    if (d->default_stop_bits == 1)
+	termctl->c_cflag &= ~(CSTOPB);
+    else
+	termctl->c_cflag |= CSTOPB;
+
+    set_termios_parity(d, termctl, find_default_int("parity"));
+    set_termios_xonoff(termctl, find_default_int("xonxoff"));
+    set_termios_rtscts(termctl, find_default_int("rtscts"));
+
+    if (find_default_int("local"))
+	termctl->c_cflag |= CLOCAL;
+    else
+	termctl->c_cflag &= ~CLOCAL;
+
+    if (find_default_int("hangup_when_done"))
+	termctl->c_cflag &= HUPCL;
+    else
+	termctl->c_cflag &= ~HUPCL;
+
+    d->disablebreak = find_default_int("nobreak");
+
     termctl->c_cflag |= CREAD;
-    termctl->c_cflag &= ~(CRTSCTS);
-    termctl->c_iflag &= ~(IXON | IXOFF | IXANY);
     termctl->c_iflag |= IGNBRK;
 }
 
@@ -297,9 +523,9 @@ devconfig(struct devcfg_data *d, struct absout *eout, const char *instr,
     char *str;
     char *pos;
     char *strtok_data;
-    int  rv = 0;
+    int  rv = 0, val;
 
-    devinit(termctl);
+    devinit(d, termctl);
 
     str = strdup(instr);
     if (str == NULL) {
@@ -308,128 +534,36 @@ devconfig(struct devcfg_data *d, struct absout *eout, const char *instr,
 
     pos = strtok_r(str, ", \t", &strtok_data);
     while (pos != NULL) {
-	if (strcmp(pos, "300") == 0) {
-	    cfsetospeed(termctl, B300);
-	    cfsetispeed(termctl, B300);
-	} else if (strcmp(pos, "600") == 0) {
-	    cfsetospeed(termctl, B600);
-	    cfsetispeed(termctl, B600);
-	} else if (strcmp(pos, "1200") == 0) {
-	    cfsetospeed(termctl, B1200);
-	    cfsetispeed(termctl, B1200);
-	} else if (strcmp(pos, "2400") == 0) {
-	    cfsetospeed(termctl, B2400);
-	    cfsetispeed(termctl, B2400);
-	} else if (strcmp(pos, "4800") == 0) {
-	    cfsetospeed(termctl, B4800);
-	    cfsetispeed(termctl, B4800);
-	} else if (strcmp(pos, "9600") == 0) {
-	    cfsetospeed(termctl, B9600);
-	    cfsetispeed(termctl, B9600);
-	} else if (strcmp(pos, "19200") == 0) {
-	    cfsetospeed(termctl, B19200);
-	    cfsetispeed(termctl, B19200);
-	} else if (strcmp(pos, "38400") == 0) {
-	    cfsetospeed(termctl, B38400);
-	    cfsetispeed(termctl, B38400);
-	} else if (strcmp(pos, "57600") == 0) {
-	    cfsetospeed(termctl, B57600);
-	    cfsetispeed(termctl, B57600);
-	} else if (strcmp(pos, "115200") == 0) {
-	    cfsetospeed(termctl, B115200);
-	    cfsetispeed(termctl, B115200);
-#ifdef B230400
-	} else if (strcmp(pos, "230400") == 0) {
-	    cfsetospeed(termctl, B230400);
-	    cfsetispeed(termctl, B230400);
-#endif
-#ifdef B460800
-	} else if (strcmp(pos, "460800") == 0) {
-	    cfsetospeed(termctl, B460800);
-	    cfsetispeed(termctl, B460800);
-#endif
-#ifdef B500000
-	} else if (strcmp(pos, "500000") == 0) {
-	    cfsetospeed(termctl, B500000);
-	    cfsetispeed(termctl, B500000);
-#endif
-#ifdef B576000
-	} else if (strcmp(pos, "576000") == 0) {
-	    cfsetospeed(termctl, B576000);
-	    cfsetispeed(termctl, B576000);
-#endif
-#ifdef B921600
-	} else if (strcmp(pos, "921600") == 0) {
-	    cfsetospeed(termctl, B921600);
-	    cfsetispeed(termctl, B921600);
-#endif
-#ifdef B1000000
-	} else if (strcmp(pos, "1000000") == 0) {
-	    cfsetospeed(termctl, B1000000);
-	    cfsetispeed(termctl, B1000000);
-#endif
-#ifdef B1152000
-	} else if (strcmp(pos, "1152000") == 0) {
-	    cfsetospeed(termctl, B1152000);
-	    cfsetispeed(termctl, B1152000);
-#endif
-#ifdef B1500000
-	} else if (strcmp(pos, "1500000") == 0) {
-	    cfsetospeed(termctl, B1500000);
-	    cfsetispeed(termctl, B1500000);
-#endif
-#ifdef B2000000
-	} else if (strcmp(pos, "2000000") == 0) {
-	    cfsetospeed(termctl, B2000000);
-	    cfsetispeed(termctl, B2000000);
-#endif
-#ifdef B2500000
-	} else if (strcmp(pos, "2500000") == 0) {
-	    cfsetospeed(termctl, B2500000);
-	    cfsetispeed(termctl, B2500000);
-#endif
-#ifdef B3000000
-	} else if (strcmp(pos, "3000000") == 0) {
-	    cfsetospeed(termctl, B3000000);
-	    cfsetispeed(termctl, B3000000);
-#endif
-#ifdef B3500000
-	} else if (strcmp(pos, "3500000") == 0) {
-	    cfsetospeed(termctl, B3500000);
-	    cfsetispeed(termctl, B3500000);
-#endif
-#ifdef B4000000
-	} else if (strcmp(pos, "4000000") == 0) {
-	    cfsetospeed(termctl, B4000000);
-	    cfsetispeed(termctl, B4000000);	    
-#endif
+	if ((val = speedstr_to_speed(pos)) != -1) {
+	    rv = set_termios_from_speed(d, termctl, val);
+	    if (rv) {
+		eout->out(eout, "Invalid baud rate: %d\n", d->default_bps);
+		return -1;
+	    }
 	} else if (strcmp(pos, "1STOPBIT") == 0) {
 	    termctl->c_cflag &= ~(CSTOPB);
+	    d->default_stop_bits = 1;
 	} else if (strcmp(pos, "2STOPBITS") == 0) {
 	    termctl->c_cflag |= CSTOPB;
+	    d->default_stop_bits = 2;
+	} else if (strcmp(pos, "5DATABITS") == 0) {
+	    set_termios_datasize(d, termctl, 5);
+	} else if (strcmp(pos, "6DATABITS") == 0) {
+	    set_termios_datasize(d, termctl, 6);
 	} else if (strcmp(pos, "7DATABITS") == 0) {
-	    termctl->c_cflag &= ~(CSIZE);
-	    termctl->c_cflag |= CS7;
+	    set_termios_datasize(d, termctl, 7);
 	} else if (strcmp(pos, "8DATABITS") == 0) {
-	    termctl->c_cflag &= ~(CSIZE);
-	    termctl->c_cflag |= CS8;
-	} else if (strcmp(pos, "NONE") == 0) {
-	    termctl->c_cflag &= ~(PARENB);
-	} else if (strcmp(pos, "EVEN") == 0) {
-	    termctl->c_cflag |= PARENB;
-	    termctl->c_cflag &= ~(PARODD);
-	} else if (strcmp(pos, "ODD") == 0) {
-	    termctl->c_cflag |= PARENB | PARODD;
+	    set_termios_datasize(d, termctl, 8);
+	} else if ((val = lookup_parity(pos)) == 0) {
+	    set_termios_parity(d, termctl, val);
         } else if (strcmp(pos, "XONXOFF") == 0) {
-            termctl->c_iflag |= (IXON | IXOFF | IXANY);
-            termctl->c_cc[VSTART] = 17;
-            termctl->c_cc[VSTOP] = 19;      
+	    set_termios_xonoff(termctl, 1);
         } else if (strcmp(pos, "-XONXOFF") == 0) {
-            termctl->c_iflag &= ~(IXON | IXOFF | IXANY);
+	    set_termios_xonoff(termctl, 0);
         } else if (strcmp(pos, "RTSCTS") == 0) {
-            termctl->c_cflag |= CRTSCTS;  
+	    set_termios_rtscts(termctl, 1);
         } else if (strcmp(pos, "-RTSCTS") == 0) {
-            termctl->c_cflag &= ~CRTSCTS;
+	    set_termios_rtscts(termctl, 0);
         } else if (strcmp(pos, "LOCAL") == 0) {
             termctl->c_cflag |= CLOCAL;  
         } else if (strcmp(pos, "-LOCAL") == 0) {
@@ -440,6 +574,8 @@ devconfig(struct devcfg_data *d, struct absout *eout, const char *instr,
             termctl->c_cflag &= ~HUPCL;
 	} else if (strcmp(pos, "NOBREAK") == 0) {
 	    d->disablebreak = 1;
+	} else if (strcmp(pos, "-NOBREAK") == 0) {
+	    d->disablebreak = 0;
 	} else {
 	    if (otherconfig(data, eout, pos) == -1)
 		goto out;
@@ -448,7 +584,7 @@ devconfig(struct devcfg_data *d, struct absout *eout, const char *instr,
 	pos = strtok_r(NULL, ", \t", &strtok_data);
     }
 
-#ifdef USE_RS485_FEATURE
+#if HAVE_DECL_TIOCSRS485
     d->conf = get_rs485_conf(data);
 #endif
  out:
@@ -712,25 +848,47 @@ do_except(int fd, void *data)
     io->except_handler(io);
 }
 
-static int devcfg_setup(struct devio *io, const char *name, const char **errstr)
+static int calc_bpc(struct devcfg_data *d)
+{
+    return d->current_data_size + d->current_stop_bits +
+	d->current_parity_on + 1;
+}
+
+static void
+devfd_fd_cleared(int fd, void *cb_data)
+{
+    struct devio *io = cb_data;
+    struct devcfg_data *d = io->my_data;
+    void (*shutdown_done)(struct devio *) = d->shutdown_done;
+
+    tcflush(d->devfd, TCOFLUSH);
+    close(d->devfd);
+    d->devfd = -1;
+    uucp_rm_lock(io->devname);
+    shutdown_done(io);
+}
+
+static int devcfg_setup(struct devio *io, const char *name, const char **errstr,
+			int *bps, int *bpc)
 {
     struct devcfg_data *d = io->my_data;
     int options;
+    int rv;
 
-#ifdef USE_UUCP_LOCKING
-    {
-	int rv;
-
-	rv = uucp_mk_lock(io->devname);
-	if (rv > 0 ) {
-	    *errstr = "Port already in use by another process\n\r";
-	    return -1;
-	} else if (rv < 0) {
-	    *errstr = "Error creating port lock file\n\r";
-	    return -1;
-	}
+    rv = uucp_mk_lock(io->devname);
+    if (rv > 0 ) {
+	*errstr = "Port already in use by another process\r\n";
+	return -1;
+    } else if (rv < 0) {
+	*errstr = "Error creating port lock file\r\n";
+	return -1;
     }
-#endif /* USE_UUCP_LOCKING */
+
+    *bps = d->current_bps = d->default_bps;
+    d->current_data_size = d->default_data_size;
+    d->current_stop_bits = d->default_stop_bits;
+    d->current_parity_on = d->default_parity_on;
+    *bpc = calc_bpc(d);
 
     /* Oct 05 2001 druzus: NOCTTY - don't make 
        device control tty for our process */
@@ -745,35 +903,31 @@ static int devcfg_setup(struct devio *io, const char *name, const char **errstr)
 	syslog(LOG_ERR, "Could not open device %s for port %s: %m",
 	       io->devname,
 	       name);
-#ifdef USE_UUCP_LOCKING
 	uucp_rm_lock(io->devname);
-#endif /* USE_UUCP_LOCKING */
 	return -1;
     }
 
-    if (!io->read_disabled && !d->disablebreak
-        && tcsetattr(d->devfd, TCSANOW, &d->termctl) == -1)
+    if (!io->read_disabled && tcsetattr(d->devfd, TCSANOW, &d->termctl) == -1)
     {
 	close(d->devfd);
 	d->devfd = -1;
 	syslog(LOG_ERR, "Could not set up device %s for port %s: %m",
 	       io->devname,
 	       name);
-#ifdef USE_UUCP_LOCKING
 	uucp_rm_lock(io->devname);
-#endif /* USE_UUCP_LOCKING */
 	return -1;
     }
 
     /* Turn off BREAK. */
-    if (!io->read_disabled && ioctl(d->devfd, TIOCCBRK) == -1) {
+    if (!io->read_disabled && !d->disablebreak
+	&& ioctl(d->devfd, TIOCCBRK) == -1) {
 	/* Probably not critical, but we should at least log something. */
 	syslog(LOG_ERR, "Could not turn off break for device %s port %s: %m",
 	       io->devname,
 	       name);
     }
 
-#ifdef USE_RS485_FEATURE
+#if HAVE_DECL_TIOCSRS485
     if (d->conf) {
         if (d->conf->flags & SER_RS485_ENABLED) {
             if (ioctl(d->devfd , TIOCSRS485, d->conf ) < 0) {
@@ -788,25 +942,23 @@ static int devcfg_setup(struct devio *io, const char *name, const char **errstr)
 
     sel_set_fd_handlers(ser2net_sel, d->devfd, io,
 			io->read_disabled ? NULL : do_read,
-			do_write, do_except);
+			do_write, do_except, devfd_fd_cleared);
     return 0;
 }
 
-static void devcfg_shutdown(struct devio *io)
+static void devcfg_shutdown(struct devio *io,
+			    void (*shutdown_done)(struct devio *))
 {
     struct devcfg_data *d = io->my_data;
 
     /* To avoid blocking on close if we have written bytes and are in
        flow-control, we flush the output queue. */
     if (d->devfd != -1) {
+	d->shutdown_done = shutdown_done;
 	sel_clear_fd_handlers(ser2net_sel, d->devfd);
-	tcflush(d->devfd, TCOFLUSH);
-	close(d->devfd);
-	d->devfd = -1;
+    } else {
+	shutdown_done(io);
     }
-#ifdef USE_UUCP_LOCKING
-    uucp_rm_lock(io->devname);
-#endif /* USE_UUCP_LOCKING */
 }
 
 static int devcfg_read(struct devio *io, void *buf, size_t size)
@@ -863,7 +1015,7 @@ static int devcfg_get_modem_state(struct devio *io, unsigned char *modemstate)
     struct devcfg_data *d = io->my_data;
     int val;
 
-    if (ioctl(d->devfd, TIOCMGET, &val) != -1)
+    if (ioctl(d->devfd, TIOCMGET, &val) != 0)
 	return -1;
 
     *modemstate = 0;
@@ -878,7 +1030,7 @@ static int devcfg_get_modem_state(struct devio *io, unsigned char *modemstate)
     return 0;
 }
 
-static int devcfg_baud_rate(struct devio *io, int *val, int cisco)
+static int devcfg_baud_rate(struct devio *io, int *val, int cisco, int *bps)
 {
     struct devcfg_data *d = io->my_data;
     struct termios termio;
@@ -888,7 +1040,7 @@ static int devcfg_baud_rate(struct devio *io, int *val, int cisco)
 	return -1;
     }
 
-    if ((*val != 0) && (get_baud_rate(*val, val, cisco))) {
+    if ((*val != 0) && (get_baud_rate(*val, val, cisco, bps))) {
 	/* We have a valid baud rate. */
 	cfsetispeed(&termio, *val);
 	cfsetospeed(&termio, *val);
@@ -902,7 +1054,7 @@ static int devcfg_baud_rate(struct devio *io, int *val, int cisco)
     return 0;
 }
 
-static int devcfg_data_size(struct devio *io, unsigned char *val)
+static int devcfg_data_size(struct devio *io, unsigned char *val, int *bpc)
 {
     struct devcfg_data *d = io->my_data;
     struct termios termio;
@@ -915,10 +1067,10 @@ static int devcfg_data_size(struct devio *io, unsigned char *val)
     if ((*val >= 5) && (*val <= 8)) {
 	termio.c_cflag &= ~CSIZE;
 	switch (*val) {
-	case 5: termio.c_cflag |= CS5; break;
-	case 6: termio.c_cflag |= CS6; break;
-	case 7: termio.c_cflag |= CS7; break;
-	case 8: termio.c_cflag |= CS8; break;
+	case 5: termio.c_cflag |= CS5; d->current_data_size = 5; break;
+	case 6: termio.c_cflag |= CS6; d->current_data_size = 6; break;
+	case 7: termio.c_cflag |= CS7; d->current_data_size = 7; break;
+	case 8: termio.c_cflag |= CS8; d->current_data_size = 8; break;
 	}
 	tcsetattr(d->devfd, TCSANOW, &termio);
     }
@@ -931,10 +1083,12 @@ static int devcfg_data_size(struct devio *io, unsigned char *val)
     default:  *val = 0;
     }
 
+    *bpc = calc_bpc(d);
+
     return 0;
 }
 
-static int devcfg_parity(struct devio *io, unsigned char *val)
+static int devcfg_parity(struct devio *io, unsigned char *val, int *bpc)
 {
     struct devcfg_data *d = io->my_data;
     struct termios termio;
@@ -948,9 +1102,13 @@ static int devcfg_parity(struct devio *io, unsigned char *val)
     if ((*val >= 1) && (*val <= 3)) {
 	termio.c_cflag &= ~(PARENB | PARODD);
 	switch (*val) {
-	case 1: break; /* NONE */
-	case 2: termio.c_cflag |= PARENB | PARODD; break; /* ODD */
-	case 3: termio.c_cflag |= PARENB; break; /* EVEN */
+	case 1: d->current_parity_on = 0; break; /* NONE */
+	case 2: termio.c_cflag |= PARENB | PARODD; /* ODD */
+	    d->current_parity_on = 1;
+	    break;
+	case 3: termio.c_cflag |= PARENB; /* EVEN */
+	    d->current_parity_on = 1;
+	    break;
 	}
 	tcsetattr(d->devfd, TCSANOW, &termio);
     }
@@ -963,10 +1121,12 @@ static int devcfg_parity(struct devio *io, unsigned char *val)
     } else
 	*val = 1; /* NONE */
 
+    *bpc = calc_bpc(d);
+
     return 0;
 }
 
-static int devcfg_stop_size(struct devio *io, unsigned char *val)
+static int devcfg_stop_size(struct devio *io, unsigned char *val, int *bpc)
 {
     struct devcfg_data *d = io->my_data;
     struct termios termio;
@@ -979,8 +1139,10 @@ static int devcfg_stop_size(struct devio *io, unsigned char *val)
     if ((*val >= 1) && (*val <= 2)) {
 	termio.c_cflag &= ~CSTOPB;
 	switch (*val) {
-	case 1: break; /* 1 stop bit */
-	case 2: termio.c_cflag |= CSTOPB; break; /* 2 stop bits */
+	case 1: d->current_stop_bits = 1; break; /* 1 stop bit */
+	case 2: d->current_stop_bits = 2; /* 2 stop bits */
+	    termio.c_cflag |= CSTOPB;
+	    break;
 	}
 	tcsetattr(d->devfd, TCSANOW, &termio);
     }
@@ -990,6 +1152,8 @@ static int devcfg_stop_size(struct devio *io, unsigned char *val)
     else
 	*val = 1; /* 1 stop bit. */
 
+    *bpc = calc_bpc(d);
+
     return 0;
 }
 
